package main

import (
	"fmt"
)

type Any interface{}
type EvalFunc func(Any) (Any, Any)

func main() {
	fibFunc := func(state Any) (Any, Any) {
		os := state.([]uint64)
		v1 := os[0]
		v2 := os[1]
		ns := []uint64{v2, v1 + v2}
		return v1, ns
	}
	fib := BuildLazyUInt64Evaluator(fibFunc, []uint64{0, 1})

	for i := 0; i < 10; i++ {
		fmt.Printf("Fib nr %v: %v\n", i, fib())
	}
}

func BuildLazyEvaluator(evalFunc EvalFunc, initState Any) func() Any {
	retValChan := make(chan Any)
	loopFunc := func() {
		var actState Any = initState
		var retVal Any
		for {
			retVal, actState = evalFunc(actState)
			retValChan <- retVal
		}
	}
	retFunc := func() Any {
		return <-retValChan
	}
	go loopFunc()
	return retFunc
}

func BuildLazyUInt64Evaluator(evalFunc EvalFunc, initState Any) func() uint64 {
	ef := BuildLazyEvaluator(evalFunc, initState)
	return func() uint64 {
		return ef().(uint64)
	}
}

/* Output:
Fib nr 0: 0
Fib nr 1: 1
Fib nr 2: 1
Fib nr 3: 2
Fib nr 4: 3
Fib nr 5: 5
Fib nr 6: 8
Fib nr 7: 13
Fib nr 8: 21
Fib nr 9: 34
*/


<div>
<div id="blogCard">
<div id="unlockReadall" class="readall-box">
<p class="read-more-mask"></p>
<a rel="noopener noreferrer" id='unlockClick' class="btn btn-lg btn-outline">阅读全文</a>
</div>
<div class="read-hot-card" style='padding: 10px 0;'>
<h2 style="margin-top:0">看完两件小事</h2>
<p class="guide-msg" >如果你觉得这篇文章对你挺有启发，我想请你帮我两个小忙：</p>
<ol>
<li style="display:none">关注我们的 <a href="https://github.com/js-china/interview-warehouse" target="_blank" rel="noopener noreferrer"><strong>GitHub 博客</strong></a>，让我们成为长期关系
</li>
<li>把这篇文章分享给你的<strong>朋友 / 交流群</strong>，让更多的人看到，一起进步，一起成长！</li>
<li>
关注公众号 <strong>「IT平头哥联盟」</strong>，公众号后台回复「<strong>资源</strong>」 免费领取我精心整理的前端进阶资源教程
</li>
</ol>
<p class="card-img">
<img src="https://static.javascriptc.com/site-common/images/javascript-card002.png" style="width: 60%;border-radius: 5px;" alt="JS中文网是中国领先的新一代开发者社区和专业的技术媒体,一个帮助开发者成长的社区，目前已经覆盖和服务了超过 300 万开发者，你每天都可以在这里找到技术世界的头条内容。欢迎热爱技术的你一起加入交流与学习,JS中文网的使命是帮助开发者用代码改变世界"/></p>
</div>
</div>
<link rel="stylesheet" href="/books/gitbook_static/custom-style.css?ver=2019.10.13"/>
<script>
var _hmt = _hmt || [];
(function() {
var hm = document.createElement("script");
hm.src = "https://hm.baidu.com/hm.js?8bee61f2b7529948168daf5e819b641c";
var s = document.getElementsByTagName("script")[0];
s.parentNode.insertBefore(hm, s);
})();
</script>
<!-- Global site tag (gtag.js) - Google Analytics -->
<script async src="https://www.googletagmanager.com/gtag/js?id=UA-126615527-1"></script>
<script>
window.dataLayer = window.dataLayer || [];
function gtag(){dataLayer.push(arguments);}
gtag('js', new Date());

gtag('config', 'UA-126615527-1');
</script>
<script src="https://www.javascriptc.com/wp-content/themes/justnews/js/jquery.min.js?ver=1.12.4"></script>
<script src="/books/gitbook_static/custom.min.js?ver=2019.11.19"></script>
</div>
